//+------------------------------------------------------------------+
//|                                      AmazingEA.mq4 Version 1.2.1 |
//|                                  1.2.1 Modified by Alan Prothero |
//|                    Added NMonth and NYear plus Broker Time Clock |
//|                                  Renamed Pips to Points Globally |
//|                                          Added Break Even Offset |
//|                                  1.2.0 Modified by Alan Prothero |
//|                            Added Settings Info in Trade Comments |
//|                                  Fixed TP=0 disables Take Profit |
//|                                  1.1.9 Modified by Alan Prothero |
//|                                            Fixed Break Even Code |
//|                    Added Order Delete Function to DeInit Routine | 
//|                                   Improved Help Text and Logging |
//|                                          alan.prothero@gmail.com |
//|             1.1.8 Modified by branac, added changeable stop loss |
//|                                                bbranac@gmail.com |
//|                                               1.1.7 FiFtHeLeMeNt |
//|                                             http://www.irxfx.com |
//|                                         fifthelement80@gmail.com |
//+------------------------------------------------------------------+
#property copyright "FiFtHeLeMeNt"
#property link      "http://www.irxfx.com"

extern int PointsAway=50; // Points Away, number of points above and below the current price where the orders will be placed
extern int TP=1000; // Take Profit, exact amount of points from open price (set to 0 to disable)
extern int SL=50; // Stop Loss, hard stop from open price (set to 0 to disable)
extern int NYear=2012;  // News Year, broker time not PC time
extern int NMonth=12;  // News Month, broker time not PC time
extern int NDay=31;  // News Day, broker time not PC time
extern int NHour=23; // News Hour, broker time not PC time
extern int NMin=59;  // News Minute, broker time not PC time
extern int CTCBN=0; // Numbers Candles To Check Before News, for determining highs & lows (when it is 1 it checks the current candle and the last one) (set to 0 to disable)
extern int SecBPO=30; // Seconds Before Pending Orders, number of seconds before news when pending orders will be placed
extern int SecBMO=2; // Seconds Before Modifying Orders, number of seconds before news when orders will stop being modified
extern int STWAN=5; // Seconds To Wait After News, number of seconds after news to wait before deleting pending orders
extern bool OCO=true; // Order Cancel Other, will cancel the other pending order if one of them is activated but only after news time
extern int BEPoints=50; // Break Even Points, number of points in profit before the stop loss is moved to Break Even + BEOffset (set to 0 to disable)
extern int BEOffset=0; // Break Even Offset, number of points to move beyond Break Even (allows to cover Broker Commissions etc.) (set to 0 to disable)
extern int TrailingStop=0; // Trailing Stop, number of points to a trailing stop (set to 0 to disable)
extern bool MM=false; // Money Management, if true uses Risk Percentage, if false uses Lot Size
extern int RiskPercent=1; // Risk Percentage, percent amount of account to risk when Money Management set to true, overrides Lot Size
extern double Lots=0.1; // Lot Size, number of lots to trade when Money Management set to false
extern string TradeLog = "AmazingEA"; // Log is created in experts/files sub-directory with this name followed by the CurrencySymbol followed by .txt as an extension
double h,l,ho,lo,hso,lso,htp,ltp,sp;
int Magic;
string filename;
string TradeComment;
string BrokerTime;

double LotsOptimized()
  {
   double lot=Lots;
  //---- select lot size
   if (MM) lot=NormalizeDouble(MathFloor(AccountFreeMargin()*RiskPercent/100)/100,1);
   
  // lot at this point is number of standard lots
   return(lot);
  } 

string AddLeadingZero(int number, int digits)
  {
   // add leading zeros that the resulting string has 'digits' length.
   string result;
   result = DoubleToStr(number, 0);
   while(StringLen(result)<digits) result = "0"+result;
   return(result);
  }
  
int CheckOrdersCondition()
  {
    int result=0;
    for (int i=0;i<OrdersTotal();i++) {
      OrderSelect(i,SELECT_BY_POS,MODE_TRADES);
      if ((OrderType()==OP_BUY) && (OrderSymbol() == Symbol()) && (OrderMagicNumber() == Magic)) {
        result=result+1000; 
      }
      if ((OrderType()==OP_SELL) && (OrderSymbol() == Symbol()) && (OrderMagicNumber() == Magic)) {
        result=result+100; 
      }
      if ((OrderType()==OP_BUYSTOP) && (OrderSymbol() == Symbol()) && (OrderMagicNumber() == Magic)) {
        result=result+10;
      }
      if ((OrderType()==OP_SELLSTOP) && (OrderSymbol() == Symbol()) && (OrderMagicNumber() == Magic)) {
        result=result+1; 
      }

    }
    return(result); // 0 means we have no trades
  }
  
// OrdersCondition Result Pattern
//    1    1    1    1
//    b    s    bs   ss
//  
  
  
void OpenBuyStop()
 {
    int ticket,err,tries;
        tries = 0;
        if (!GlobalVariableCheck("InTrade")) {
          while (tries < 3)
            {
               GlobalVariableSet("InTrade", CurTime());  // set lock indicator
               ticket = OrderSend(Symbol(),OP_BUYSTOP,LotsOptimized(),ho,1,hso,htp,TradeComment,Magic,0,Green);
               Write("OpenBuyStop, OrderSend Executed, ticket="+ticket);
               GlobalVariableDel("InTrade");   // clear lock indicator
               if(ticket<=0) {
                  Write("Error Occurred : "+ErrorDescription(GetLastError())+" BuyStop @ "+ho+" SL @ "+hso+" TP @"+htp);
                  tries++;
               } else tries = 3;
            }
        }
 }
  
void OpenSellStop()
 {
    int ticket,err,tries;
        tries = 0;
        if (!GlobalVariableCheck("InTrade")) {
          while (tries < 3)
            {
               GlobalVariableSet("InTrade", CurTime());  // set lock indicator
               ticket = OrderSend(Symbol(),OP_SELLSTOP,LotsOptimized(),lo,1,lso,ltp,TradeComment,Magic,0,Red);
               Write("OpenSellStop, OrderSend Executed, ticket="+ticket);
               GlobalVariableDel("InTrade");   // clear lock indicator
               if(ticket<=0) {
                  Write("Error Occurred : "+ErrorDescription(GetLastError())+" SellStop @ "+lo+" SL @ "+lso+" TP @"+ltp);
                  tries++;
               } else tries = 3;
            }
        }
 }
 
void DoBE()
  {
    for (int i = 0; i < OrdersTotal(); i++) {
     OrderSelect(i, SELECT_BY_POS, MODE_TRADES);
     if ( OrderSymbol()==Symbol() && (OrderMagicNumber() == Magic))  // only look if mygrid and symbol...
        {
            if (OrderType() == OP_BUY) {
               if (Bid - OrderOpenPrice() > BEPoints * Point)
               {
                  if (OrderStopLoss() < OrderOpenPrice()) 
                  {
                     Write("Moving Stop Loss of Buy Order to Break Even plus BEOffset");
                     OrderModify(OrderTicket(), OrderOpenPrice(), OrderOpenPrice() + (BEOffset * Point), OrderTakeProfit(), Blue);
                  }
               }
            }
            if (OrderType() == OP_SELL){
               if (OrderOpenPrice() - Ask > BEPoints * Point) 
               {
                  if (OrderStopLoss() > OrderOpenPrice()) 
                  { 
                     Write("Moving Stop Loss of Sell Order to Break Even plus BEOffset");
                     OrderModify(OrderTicket(), OrderOpenPrice(), OrderOpenPrice() - (BEOffset * Point), OrderTakeProfit(), Blue);
                  }
               }
            }
        }
    }
  }

void DoTrail()
  {
    for (int i = 0; i < OrdersTotal(); i++) {
     OrderSelect(i, SELECT_BY_POS, MODE_TRADES);
     if ( OrderSymbol()==Symbol() && (OrderMagicNumber() == Magic))  // only look if mygrid and symbol...
        {
          
          if (OrderType() == OP_BUY) {
             if(Bid-OrderOpenPrice()>Point*TrailingStop)
             {
                if(OrderStopLoss()<Bid-Point*TrailingStop)
                  {
                     Write("Trailing Stop Loss of Buy Order set to "+DoubleToStr(Bid-Point*TrailingStop,5));
                     OrderModify(OrderTicket(),OrderOpenPrice(),Bid-Point*TrailingStop,OrderTakeProfit(),0,Green);
                  }
             }
          }

          if (OrderType() == OP_SELL) 
          {
             if((OrderOpenPrice()-Ask)>(Point*TrailingStop))
             {
                if((OrderStopLoss()>(Ask+Point*TrailingStop)) || (OrderStopLoss()==0))
                {
                     Write("Trailing Stop Loss of Sell Order set to "+DoubleToStr(Ask+Point*TrailingStop,5));
                     OrderModify(OrderTicket(),OrderOpenPrice(),Ask+Point*TrailingStop,OrderTakeProfit(),0,Red);
                }
             }
          }
       }
    }
 }
 

void DeleteBuyStop()
{
   for (int i = 0; i < OrdersTotal(); i++) {
     OrderSelect(i, SELECT_BY_POS, MODE_TRADES);
     if (OrderSymbol()==Symbol() && (OrderMagicNumber() == Magic) && (OrderType()==OP_BUYSTOP)) {
       OrderDelete(OrderTicket());
       Write("DeleteBuyStop, OrderDelete Executed");
     }
       
   }
}
   
void DeleteSellStop()
{
   for (int i = 0; i < OrdersTotal(); i++) {
     OrderSelect(i, SELECT_BY_POS, MODE_TRADES);
     if (OrderSymbol()==Symbol() && (OrderMagicNumber() == Magic) && (OrderType()==OP_SELLSTOP)) {
       OrderDelete(OrderTicket());
       Write("DeleteSellStop, OrderDelete Executed");
     }
       
   }
}

void DoModify()
{
   for (int i = 0; i < OrdersTotal(); i++) {
     OrderSelect(i, SELECT_BY_POS, MODE_TRADES);
     if (OrderSymbol()==Symbol() && (OrderMagicNumber() == Magic) && (OrderType()==OP_SELLSTOP)) {
       if ((OrderOpenPrice()>lo) || (OrderOpenPrice()<lo)) {
         Write("Sell Stop was @ "+DoubleToStr(OrderOpenPrice(),5)+", changed to "+DoubleToStr(lo,5));
         OrderModify(OrderTicket(),lo,lso,ltp,0,Red);
       }
     }

     if (OrderSymbol()==Symbol() && (OrderMagicNumber() == Magic) && (OrderType()==OP_BUYSTOP)) {
       if ((OrderOpenPrice()>ho) || (OrderOpenPrice()<ho)) {
         Write("Buy Stop was @ "+DoubleToStr(OrderOpenPrice(),5)+", changed to "+DoubleToStr(ho,5));
         OrderModify(OrderTicket(),ho,hso,htp,0,Green);
       }
     }
   }
}

int Write(string str)
{
   int handle;
  
   handle = FileOpen(filename,FILE_READ|FILE_WRITE|FILE_CSV,"/t");
   FileSeek(handle, 0, SEEK_END);      
   FileWrite(handle, TimeToStr(CurTime(),TIME_DATE|TIME_SECONDS) + " " + str);
   FileClose(handle);
}

void OrdersDeleteAll()
{
   Write("Deleting BuyStop and SellStop because EA shut down");
         DeleteBuyStop();
         DeleteSellStop();
}

//+------------------------------------------------------------------+
//| expert initialization function                                   |
//+------------------------------------------------------------------+
int init()
  {
    ObjectsDeleteAll();
    return(0);
  }
  
//+------------------------------------------------------------------+
//| expert deinitialization function                                 |
//+------------------------------------------------------------------+
int deinit()
  {
    Comment("");
    OrdersDeleteAll();
    ObjectsDeleteAll();
    return(0);
  }

//+------------------------------------------------------------------+
//| expert start function                                            |
//+------------------------------------------------------------------+
int start()
  {
  
   Magic=NDay+NHour+NMin;
   
   int i;
   int OrdersCondition,secofday,secofnews;
      
   filename=TradeLog + "-" + Symbol() + ".txt";

   if (BEPoints>0) DoBE();
   
   if (TrailingStop>0) DoTrail();
   
   OrdersCondition=CheckOrdersCondition();
   
   secofday=Hour()*3600+Minute()*60+Seconds();
   secofnews=NHour*3600+NMin*60;
    
   
   h=iHigh(NULL,PERIOD_M1,0);
   l=iLow(NULL,PERIOD_M1,0);
   for (i=1;i<=CTCBN;i++) if (iHigh(NULL,PERIOD_M1,i)>h) h=iHigh(NULL,PERIOD_M1,i);
   for (i=1;i<=CTCBN;i++) if (iLow(NULL,PERIOD_M1,i)<l) l=iLow(NULL,PERIOD_M1,i);
   sp=Ask-Bid;
   ho=h+sp+(PointsAway)*Point;
   lo=l-(PointsAway)*Point;
   if (SL==0)
   {
      hso = 0;
      lso = 0;
   }
   else
   {
      hso=Ask+(PointsAway-SL)*Point; //hso=h+sp;
      lso=Bid-(PointsAway-SL)*Point; //lso=l;
   }
   if (TP==0)
   {
      htp = 0;
      ltp = 0;
   }
   else
   { 
      htp=ho+TP*Point;
      ltp=lo-TP*Point;
   }
   
   string newstime=TimeToStr(secofnews,TIME_SECONDS);
   
   string DisplayMM="False";
      if (MM) DisplayMM="True";
      
   string DisplayOCO="False";
      if (OCO) DisplayOCO="True";
   
   // TradeComment gets added in the Comment field of trades. Max 32 chars.
   // Pipe character denotes different section, helping to logically group parameters.
   TradeComment=StringConcatenate("P", PointsAway, "S", SL, "T", TP, "|P", SecBPO, "M", SecBMO, "W", STWAN, "|B", BEPoints, "T", TrailingStop);
   
   // BrokerTime is shown in the EA Comment.
   BrokerTime=TimeToStr(TimeCurrent(),TIME_DATE|TIME_SECONDS);
   
   if (Day()==NDay && Month()==NMonth && Year()==NYear) 
   {
   Comment("\nAmazing Forex System Expert Advisor v 1.2.1 By FiFtHeLeMeNt, Amended by Alan Prothero\n\nBroker Time : ", BrokerTime, "\nNews Time : ",NYear,".",AddLeadingZero(NMonth,2),".",AddLeadingZero(NDay,2)," ",newstime,"\n\nHigh @ ",DoubleToStr(h+sp,5)," BuyOrder @ ",DoubleToStr(ho,5)," StopLoss @ ",DoubleToStr(hso,5)," TakeProfit @ ",DoubleToStr(htp,5),"\nLow @ ",DoubleToStr(l,5)," SellOrder @ ",DoubleToStr(lo,5)," StopLoss @ ",DoubleToStr(lso,5)," TakeProfit @ ",DoubleToStr(ltp,5),"\n\nPointsAway : ",PointsAway," BEPoints : ",BEPoints," TrailingStop : ", TrailingStop," CTCBN : ",CTCBN," SecBPO : ",SecBPO," SecBMO : ",SecBMO," STWAN : ",STWAN," OCO : ",DisplayOCO,"\n\nMoney Management : ",DisplayMM," RiskPercent: ",RiskPercent," Lots : ",LotsOptimized());
   }
   else
   {
   Comment("\nAmazing Forex System Expert Advisor v 1.2.1 By FiFtHeLeMeNt, Amended by Alan Prothero\n\nBroker Time : ", BrokerTime, "\nNews Time : ",NYear,".",AddLeadingZero(NMonth,2),".",AddLeadingZero(NDay,2)," ",newstime,"\n\nExpert is disabled because it is not day of expected news");
   // Despite the comment above, the expert is not really disabled, it just exits without trading.
   // The return statement below is very important as it ensures the EA exits without opening trades on non-news days. 
   return(0);
   } 
   
   if ((secofday<secofnews) && (secofday>(secofnews-SecBPO))) {
      
      if (OrdersCondition==0) {
         Write("Opening BuyStop & SellStop, OrdersCondition="+OrdersCondition+" SecOfDay="+secofday);
         OpenBuyStop();
         OpenSellStop();
      }

      if (OrdersCondition==10) {
         Write("Opening SellStop, OrdersCondition="+OrdersCondition+" SecOfDay="+secofday);
         OpenSellStop();
      }
      
      if (OrdersCondition==1) {
         Write("Opening BuyStop, OrdersCondition="+OrdersCondition+" SecOfDay="+secofday);
         OpenBuyStop();
      }
   }

   if ((secofday<secofnews) && (secofday>(secofnews-SecBPO)) && (secofday<(secofnews-SecBMO))) {
         Write("Modifying Orders, OrdersCondition="+OrdersCondition+" SecOfDay="+secofday);
         DoModify();
      }
      
   


   if ((secofday>secofnews) && (secofday<(secofnews+STWAN)) && OCO) {

      if (OrdersCondition==1001) {
         Write("Deleting SellStop because BuyStop Hit, OrdersCondition="+OrdersCondition+" SecOfDay="+secofday);
         DeleteSellStop();
      }
      
      if (OrdersCondition==110) {
        Write("Deleting BuyStop because SellStop Hit, OrdersCondition="+OrdersCondition+" SecOfDay="+secofday);
        DeleteBuyStop();
      }
   }
   
   if ((secofday>secofnews) && (secofday>(secofnews+STWAN))) {
      if (OrdersCondition==11) {
         Write("Deleting BuyStop and SellStop because STWAN expired, OrdersCondition="+OrdersCondition+" SecOfDay="+secofday);
         DeleteBuyStop();
         DeleteSellStop();
      }
      
      if ((OrdersCondition==10) || (OrdersCondition==110)) {
        Write("Deleting BuyStop because STWAN expired, OrdersCondition="+OrdersCondition+" SecOfDay="+secofday);
        DeleteBuyStop();
      }
      
      if ((OrdersCondition==1) || (OrdersCondition==1001)) {
        Write("Deleting SellStop because STWAN expired, OrdersCondition="+OrdersCondition+" SecOfDay="+secofday);
        DeleteSellStop();
      }
   }
        
   
//----
   return(0);
  }
  
//+------------------------------------------------------------------+
//| return error description                                         |
//+------------------------------------------------------------------+
string ErrorDescription(int error_code)
  {
   string error_string;
//----
   switch(error_code)
     {
      //---- codes returned from trade server
      case 0:
      case 1:   error_string="no error";                                                  break;
      case 2:   error_string="common error";                                              break;
      case 3:   error_string="invalid trade parameters";                                  break;
      case 4:   error_string="trade server is busy";                                      break;
      case 5:   error_string="old version of the client terminal";                        break;
      case 6:   error_string="no connection with trade server";                           break;
      case 7:   error_string="not enough rights";                                         break;
      case 8:   error_string="too frequent requests";                                     break;
      case 9:   error_string="malfunctional trade operation";                             break;
      case 64:  error_string="account disabled";                                          break;
      case 65:  error_string="invalid account";                                           break;
      case 128: error_string="trade timeout";                                             break;
      case 129: error_string="invalid price";                                             break;
      case 130: error_string="invalid stops";                                             break;
      case 131: error_string="invalid trade volume";                                      break;
      case 132: error_string="market is closed";                                          break;
      case 133: error_string="trade is disabled";                                         break;
      case 134: error_string="not enough money";                                          break;
      case 135: error_string="price changed";                                             break;
      case 136: error_string="off quotes";                                                break;
      case 137: error_string="broker is busy";                                            break;
      case 138: error_string="requote";                                                   break;
      case 139: error_string="order is locked";                                           break;
      case 140: error_string="long positions only allowed";                               break;
      case 141: error_string="too many requests";                                         break;
      case 145: error_string="modification denied because order too close to market";     break;
      case 146: error_string="trade context is busy";                                     break;
      //---- mql4 errors
      case 4000: error_string="no error";                                                 break;
      case 4001: error_string="wrong function pointer";                                   break;
      case 4002: error_string="array index is out of range";                              break;
      case 4003: error_string="no memory for function call stack";                        break;
      case 4004: error_string="recursive stack overflow";                                 break;
      case 4005: error_string="not enough stack for parameter";                           break;
      case 4006: error_string="no memory for parameter string";                           break;
      case 4007: error_string="no memory for temp string";                                break;
      case 4008: error_string="not initialized string";                                   break;
      case 4009: error_string="not initialized string in array";                          break;
      case 4010: error_string="no memory for array\' string";                             break;
      case 4011: error_string="too long string";                                          break;
      case 4012: error_string="remainder from zero divide";                               break;
      case 4013: error_string="zero divide";                                              break;
      case 4014: error_string="unknown command";                                          break;
      case 4015: error_string="wrong jump (never generated error)";                       break;
      case 4016: error_string="not initialized array";                                    break;
      case 4017: error_string="dll calls are not allowed";                                break;
      case 4018: error_string="cannot load library";                                      break;
      case 4019: error_string="cannot call function";                                     break;
      case 4020: error_string="expert function calls are not allowed";                    break;
      case 4021: error_string="not enough memory for temp string returned from function"; break;
      case 4022: error_string="system is busy (never generated error)";                   break;
      case 4050: error_string="invalid function parameters count";                        break;
      case 4051: error_string="invalid function parameter value";                         break;
      case 4052: error_string="string function internal error";                           break;
      case 4053: error_string="some array error";                                         break;
      case 4054: error_string="incorrect series array using";                             break;
      case 4055: error_string="custom indicator error";                                   break;
      case 4056: error_string="arrays are incompatible";                                  break;
      case 4057: error_string="global variables processing error";                        break;
      case 4058: error_string="global variable not found";                                break;
      case 4059: error_string="function is not allowed in testing mode";                  break;
      case 4060: error_string="function is not confirmed";                                break;
      case 4061: error_string="send mail error";                                          break;
      case 4062: error_string="string parameter expected";                                break;
      case 4063: error_string="integer parameter expected";                               break;
      case 4064: error_string="double parameter expected";                                break;
      case 4065: error_string="array as parameter expected";                              break;
      case 4066: error_string="requested history data in update state";                   break;
      case 4099: error_string="end of file";                                              break;
      case 4100: error_string="some file error";                                          break;
      case 4101: error_string="wrong file name";                                          break;
      case 4102: error_string="too many opened files";                                    break;
      case 4103: error_string="cannot open file";                                         break;
      case 4104: error_string="incompatible access to a file";                            break;
      case 4105: error_string="no order selected";                                        break;
      case 4106: error_string="unknown symbol";                                           break;
      case 4107: error_string="invalid price parameter for trade function";               break;
      case 4108: error_string="invalid ticket";                                           break;
      case 4109: error_string="trade is not allowed";                                     break;
      case 4110: error_string="longs are not allowed";                                    break;
      case 4111: error_string="shorts are not allowed";                                   break;
      case 4200: error_string="object is already exist";                                  break;
      case 4201: error_string="unknown object property";                                  break;
      case 4202: error_string="object is not exist";                                      break;
      case 4203: error_string="unknown object type";                                      break;
      case 4204: error_string="no object name";                                           break;
      case 4205: error_string="object coordinates error";                                 break;
      case 4206: error_string="no specified subwindow";                                   break;
      default:   error_string="unknown error";
     }
//----
   return(error_string);
  }  
//+------------------------------------------------------------------+